// ignore_for_file: cancel_subscriptions, public_member_api_docs
// ignore_for_file: non_constant_identifier_names, empty_catches
// ignore_for_file: avoid_dynamic_calls

import 'package:healthrian_ble_support/healthrian_ble_support.dart';
import 'package:healthrian_ble_support_for_cardioclick/'
    'healthrian_ble_support_for_cardioclick.dart' as CardioClick;
import 'package:healthrian_ble_support_for_wearcardiot/'
    'healthrian_ble_support_for_wearcardiot.dart' as WaerCardioT;
import 'package:healthrian_ble_support_for_wearecg12/'
    'healthrian_ble_support_for_wearecg12.dart' as WearEcg12;

class BLEManager extends ManagerInterface {
  factory BLEManager() {
    return _instance;
  }

  BLEManager._internal() {}

  static final BLEManager _instance = BLEManager._internal();

  final enable = [true, true, true, false];

  /// higher weight would be early processed
  final priority = [0, 0, 0, 0];

  late final WearEcg12.BLEManager wearEcg12 = WearEcg12.BLEManager();
  late final CardioClick.BLEManager cardioClick = CardioClick.BLEManager();
  late final WaerCardioT.BLEManager wearCardioT = WaerCardioT.BLEManager();

  ModuleCustom? moduleCustom;

  ManagerCustom? get custom =>
      moduleCustom == null ? null : ManagerCustom(module: moduleCustom!);

  // final modules = <String, ModuleInterface>{};
  //
  // void saveModule(ModuleInterface module){
  //   modules[module.ble.device!.platformName] = module;
  // }
  //
  // ModuleInterface? loadModule(String deviceName) {
  //   return modules[deviceName];
  // }

  late final ModuleInterface scanner = ModuleCustom(
    ble: ModuleBle(
      uuid: UUID(),
      setDevice: (device) {
        final sorted = priority.indexed.toList()..sort((a, b) => b.$2 - a.$2);
        final list = [
          () {
            if (!enable[0]) return false;
            if (!wearEcg12.scanFilter(device)) return false;
            wearEcg12.device = device;
            return true;
          },
          () {
            if (!enable[1]) return false;
            if (!cardioClick.scanFilter(device)) return false;
            cardioClick.device = device;
            return true;
          },
          () {
            if (!enable[2]) return false;
            if (!wearCardioT.scanFilter(device)) return false;
            wearCardioT.device = device;
            return true;
          },
          () {
            if (!enable[3]) return false;
            if (custom?.scanFilter(device) != true) return false;
            custom?.device = device;
            return true;
          },
        ];
        sorted.firstWhere((e) => list[e.$1].call());
      },
      scanFilter: (device) {
        final wearEcg = Builtin.scanFilterWearEcg(device);
        final cardioClick = Builtin.scanFilterCardioClick(device);
        final wearCardioT = Builtin.scanFilterWearCardioT(device);
        final customScan = custom?.scanFilter(device) ?? false;

        if (wearEcg && enable[0]) return true;
        if (cardioClick && enable[1]) return true;
        if (wearCardioT && enable[2]) return true;
        if (customScan && enable[3]) return true;
        return false;
      },
    ),
  );

  bool get _canWearEcg12 => wearEcg12.device != null && enable[0];

  bool get _canCardioClick => cardioClick.device != null && enable[1];

  bool get _canWearCardioT => wearCardioT.device != null && enable[2];

  bool get _canCustom => custom?.device != null && enable[3];

  @override
  ModuleInterface get module {
    final modules = [
      (_canWearEcg12, wearEcg12.module),
      (_canCardioClick, cardioClick.module),
      (_canWearCardioT, wearCardioT.module),
      (_canCustom, custom?.module)
    ];
    final sorted = priority.indexed.toList()..sort((a, b) => b.$2 - a.$2);
    final selected = sorted
        .map((e) => modules[e.$1].$1 ? modules[e.$1].$2 : null)
        .firstOrNull;

    if (selected != null) return selected;
    return scanner;
  }

  @override
  Future<bool> connectDevice([
    String preferredDeviceId = '',
    bool strictConnection = false,
  ]) {
    return scanner.ble.connectDevice(preferredDeviceId, strictConnection);
  }

  @deprecated
  get manageWearEcg12 => wearEcg12;

  @deprecated
  get manageCardioClick => cardioClick;

  @deprecated
  get manageWearCardioT => wearCardioT;

  @deprecated
  get manageCustom => custom;

  @override
  ModuleInterface createModule() => module;
}
