import 'package:healthrian_ble_support/healthrian_ble_support.dart';
import 'package:healthrian_ble_support_for_wearcardiot/'
    'healthrian_ble_support_for_wearcardiot.dart' as WaerCardioT;
import 'package:healthrian_ble_support_for_wearecg12/'
    'healthrian_ble_support_for_wearecg12.dart' as WearEcg12;
import 'package:healthrian_ble_support_for_cardioclick/'
    'healthrian_ble_support_for_cardioclick.dart' as CardioClick;

class BLEMultiManager extends ManagerInterface {
  factory BLEMultiManager() {
    return _instance;
  }

  BLEMultiManager._internal() {}

  static final BLEMultiManager _instance = BLEMultiManager._internal();

  final enable = [true, true, true, false];

  /// higher weight would be early processed
  final priority = [0, 0, 0, 0];

  late final WearEcg12.BLEManager wearEcg12 = WearEcg12.BLEManager();
  late final CardioClick.BLEManager cardioClick = CardioClick.BLEManager();
  late final WaerCardioT.BLEManager wearCardioT = WaerCardioT.BLEManager();

  ModuleCustom? moduleCustom;

  ManagerCustom? get custom =>
      moduleCustom == null ? null : ManagerCustom(module: moduleCustom!);

  late final ModuleInterface scanner = ModuleCustom(
    ble: ModuleBle(
      uuid: UUID(),
      setDevice: (device) {
        final sorted = priority.indexed.toList()..sort((a, b) => b.$2 - a.$2);
        final list = [
              () {
            if (!enable[0]) return false;
            if (!wearEcg12.scanFilter(device)) return false;
            wearEcg12.device = device;
            return true;
          },
              () {
            if (!enable[1]) return false;
            if (!cardioClick.scanFilter(device)) return false;
            cardioClick.device = device;
            return true;
          },
              () {
            if (!enable[2]) return false;
            if (!wearCardioT.scanFilter(device)) return false;
            wearCardioT.device = device;
            return true;
          },
              () {
            if (!enable[3]) return false;
            if (custom?.scanFilter(device) != true) return false;
            custom?.device = device;
            return true;
          },
        ];
        sorted.firstWhere((e) => list[e.$1].call());
      },
      scanFilter: (device) {
        final wearEcg = Builtin.scanFilterWearEcg(device);
        final cardioClick = Builtin.scanFilterCardioClick(device);
        final wearCardioT = Builtin.scanFilterWearCardioT(device);
        final customScan = custom?.scanFilter(device) ?? false;

        if (wearEcg && enable[0]) return true;
        if (cardioClick && enable[1]) return true;
        if (wearCardioT && enable[2]) return true;
        if (customScan && enable[3]) return true;
        return false;
      },
    ),
  );

  @override
  ModuleInterface createModule()  => module;
}